@@ -114,7 +114,7 @@ AbstractStream::AbstractStream(const AVFormatContext *formatContext,
     auto optKey = QString("%1/%2/%3").arg(formatContext->oformat->name)
                                      .arg(streamIndex)
                                      .arg(codecName);
-    QVariantMap options = codecOptions.value(optKey);
+    auto options = codecOptions.value(optKey);
 
     if (codecName == "libvpx") {
         if (!options.contains("deadline"))
@@ -127,9 +127,7 @@ AbstractStream::AbstractStream(const AVFormatContext *formatContext,
             options["preset"] = "ultrafast";
     }
 
-    for (auto it = options.begin();
-         it != options.end();
-         it++) {
+    for (auto it = options.begin(); it != options.end(); it++) {
         QString value = it.value().toString();
 
         av_dict_set(&this->d->m_codecOptions,
@@ -204,12 +202,12 @@ void AbstractStream::packetEnqueue(const AkPacket &packet)
 
 void AbstractStream::convertPacket(const AkPacket &packet)
 {
-    Q_UNUSED(packet);
+    Q_UNUSED(packet)
 }
 
 int AbstractStream::encodeData(AVFrame *frame)
 {
-    Q_UNUSED(frame);
+    Q_UNUSED(frame)
 
     return AVERROR_EOF;
 }
@@ -308,7 +306,6 @@ bool AbstractStream::init()
 
 void AbstractStream::uninit()
 {
-
     this->d->m_runConvertLoop = false;
     waitLoop(this->d->m_convertLoopResult);
 
