@@ -22,6 +22,8 @@
 
 #include <QObject>
 
+class MultiSinkGlobalsPrivate;
+
 class MultiSinkGlobals: public QObject
 {
     Q_OBJECT
@@ -30,15 +32,18 @@ class MultiSinkGlobals: public QObject
                WRITE setCodecLib
                RESET resetCodecLib
                NOTIFY codecLibChanged)
+    Q_PROPERTY(QStringList subModules
+               READ subModules)
 
     public:
         MultiSinkGlobals(QObject *parent=nullptr);
+        ~MultiSinkGlobals();
 
         Q_INVOKABLE QString codecLib() const;
+        Q_INVOKABLE QStringList subModules() const;
 
     private:
-        QString m_codecLib;
-        QStringList m_preferredFramework;
+        MultiSinkGlobalsPrivate *d;
 
     signals:
         void codecLibChanged(const QString &codecLib);
