@@ -25,34 +25,29 @@
 template<typename T>
 struct Pixel
 {
-    Pixel():
-        red(0), green(0), blue(0), alpha(0)
+    T r;
+    T g;
+    T b;
+    T a;
+
+    inline Pixel(T r=0, T g=0, T b=0, T a=0):
+        r(r), g(g), b(b), a(a)
     {
     }
 
-    Pixel(T red, T green, T blue, T alpha):
-        red(red), green(green), blue(blue), alpha(alpha)
-    {
+    inline void clear() {
+        this->r = 0;
+        this->g = 0;
+        this->b = 0;
+        this->a = 0;
     }
-
-    void clear() {
-        this->red = 0;
-        this->green = 0;
-        this->blue = 0;
-        this->alpha = 0;
-    }
-
-    T red;
-    T green;
-    T blue;
-    T alpha;
 };
 
 // These are used as accumulators
-typedef struct Pixel<qint32> IntegerPixel;
-typedef struct Pixel<quint32> UIntegerPixel;
-typedef struct Pixel<quint16> ShortPixel;
-typedef struct Pixel<quint8>  CharPixel;
+typedef Pixel<qint32> IntegerPixel;
+typedef Pixel<quint32> UIntegerPixel;
+typedef Pixel<quint16> ShortPixel;
+typedef Pixel<quint8> CharPixel;
 typedef IntegerPixel HistogramListItem;
 
 #endif // PIXELSTRUCTS_H
