@@ -29,6 +29,9 @@ class AkCaps;
 class VirtualCameraElement: public AkElement
 {
     Q_OBJECT
+    Q_PROPERTY(QString errorMessage
+               READ errorMessage
+               NOTIFY errorMessageChanged)
     Q_PROPERTY(QStringList driverPaths
                READ driverPaths
                WRITE setDriverPaths
@@ -69,6 +72,7 @@ class VirtualCameraElement: public AkElement
         VirtualCameraElement();
         ~VirtualCameraElement();
 
+        Q_INVOKABLE QString errorMessage() const;
         Q_INVOKABLE QStringList driverPaths() const;
         Q_INVOKABLE QStringList medias() const;
         Q_INVOKABLE QString media() const;
@@ -100,8 +104,10 @@ class VirtualCameraElement: public AkElement
         QString controlInterfaceProvide(const QString &controlId) const;
         void controlInterfaceConfigure(QQmlContext *context,
                                        const QString &controlId) const;
+        AkPacket iVideoStream(const AkVideoPacket &packet);
 
     signals:
+        void errorMessageChanged(const QString &error);
         void driverPathsChanged(const QStringList &driverPaths);
         void mediasChanged(const QStringList &medias) const;
         void mediaChanged(const QString &media);
@@ -111,7 +117,6 @@ class VirtualCameraElement: public AkElement
         void availableDriversChanged(const QStringList &availableDrivers);
         void rootMethodChanged(const QString &rootMethod);
         void availableMethodsChanged(const QStringList &availableMethods);
-        void error(const QString &message);
 
     public slots:
         void setDriverPaths(const QStringList &driverPaths);
@@ -129,7 +134,6 @@ class VirtualCameraElement: public AkElement
         void clearStreams();
 
         bool setState(AkElement::ElementState state);
-        AkPacket iStream(const AkPacket &packet);
 
     private slots:
         void rootMethodUpdated(const QString &rootMethod);
