@@ -18,16 +18,30 @@
 #
 # Web-Site: http://webcamoid.github.io/
 
+[ -f environment.sh ] && source environment.sh
+
 pacman -Syy
 pacman --noconfirm --needed -S \
     git \
     make \
     pkg-config \
-    python3 \
-    mingw-w64-x86_64-pkg-config \
-    mingw-w64-x86_64-qt5 \
-    mingw-w64-x86_64-ffmpeg \
-    mingw-w64-x86_64-gst-plugins-base \
-    mingw-w64-x86_64-gst-plugins-good \
-    mingw-w64-x86_64-gst-plugins-bad \
-    mingw-w64-x86_64-gst-plugins-ugly
+    python3
+
+if [ "${PLATFORM}" = x86 ]; then
+    packagesArch=i686
+else
+    packagesArch=x86_64
+fi
+
+pacman --noconfirm --needed -S \
+    mingw-w64-${packagesArch}-pkg-config \
+    mingw-w64-${packagesArch}-qt5 \
+    mingw-w64-${packagesArch}-ffmpeg
+
+if [ -z "${DAILY_BUILD}" ] && [ -z "${RELEASE_BUILD}" ]; then
+    pacman --noconfirm --needed -S \
+        mingw-w64-${packagesArch}-gst-plugins-base \
+        mingw-w64-${packagesArch}-gst-plugins-good \
+        mingw-w64-${packagesArch}-gst-plugins-bad \
+        mingw-w64-${packagesArch}-gst-plugins-ugly
+fi
