@@ -26,7 +26,7 @@ pkg install -y \
     qt5-qmake \
     qt5-buildtools \
     qt5-concurrent \
-    qt5-multimedia \
+    qt5-opengl \
     qt5-quickcontrols \
     qt5-quickcontrols2 \
     qt5-svg \
